@@ -1,6 +1,7 @@
 import base64
 import collections
 import hashlib
+import json
 import logging
 import progressbar
 import re
@@ -15,9 +16,7 @@ from filtercascade import FilterCascade
 from moz_crlite_lib import CertId, IssuerId, readFromAdditionsList
 from pathlib import Path
 from pyasn1.codec.der.decoder import decode as der_decoder
-from pyasn1.type import namedtype
-from pyasn1.type import tag
-from pyasn1.type import univ
+from pyasn1.type import namedtype, tag, univ
 from pyasn1_modules import pem
 from pyasn1_modules import rfc2459
 from urllib.parse import urljoin
@@ -341,15 +340,6 @@ class CRLiteDB(object):
         return results
 
 
-# Theoretically this should get us the raw serial, but it... doesn't.
-# class RawSerial(univ.OctetString):
-#     tagSet = tag.initTagSet(
-#         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x02)
-#     )
-#     subtypeSpec = constraint.ConstraintsIntersection()
-#     typeId = univ.OctetString.getTypeId()
-
-
 class RawTBSCertificate(univ.Sequence):
     componentType = namedtype.NamedTypes(
         namedtype.DefaultedNamedType(
@@ -419,6 +409,21 @@ class CRLiteQueryResult(object):
 
         rev_status = crlite_db.revocation_status(self.cert_id)
 
+        if not rev_status["revoked"] and self.cert_id.serial[0] & 0b10000000:
+            # Try again if the first byte of the serial might have had a leading
+            # zero stripped off by pyasn.1.
+            tmp_serial = self.cert_id.serial
+            serial_with_leading_zero = tmp_serial.rjust(1 + len(tmp_serial), b"\0")
+            cert_id_with_leading_zero = CertId(
+                issuer["issuerId"], serial_with_leading_zero
+            )
+            log.debug(
+                f"Received a not-revoked answer for serial {self.cert_id}, "
+                + "but there could be a missing leading zero, so retrying with "
+                + f"serial {cert_id_with_leading_zero}"
+            )
+            rev_status = crlite_db.revocation_status(cert_id_with_leading_zero)
+
         if rev_status["revoked"] is True:
             if "via_stash" in rev_status:
                 self.via_stash = rev_status["via_stash"]
@@ -450,7 +455,7 @@ class CRLiteQueryResult(object):
             return "⁉️"
         return ""
 
-    def print_query_result(self):
+    def print_query_result(self, *, verbose=0):
         padded_name = self.name + " " * 5
         padding = "".ljust(len(padded_name))
 
@@ -461,7 +466,8 @@ class CRLiteQueryResult(object):
 
             print(f"{padded_name} Issuer: {self.issuer['subject']}")
             print(f"{padding} Enrolled in CRLite: {enrolled_icon}")
-            print(f"{padding} {self.cert_id}")
+            if verbose > 0:
+                print(f"{padding} {self.cert_id}")
             if self.via_filter:
                 print(f"{padding} Revoked via CRLite filter: {self.via_filter}")
             if self.via_stash:
@@ -480,17 +486,16 @@ class CRLiteQueryResult(object):
             "name": self.name,
             "issuer": self.issuer["subject"],
             "state": self.state,
-            "state_icon": self.result_icon(),
-            "enrolled_in_crlite": "✅" if self.issuer["crlite_enrolled"] else "❌",
-            "cert_id": self.cert_id,
+            "enrolled_in_crlite": self.issuer["crlite_enrolled"],
+            "cert_id": str(self.cert_id),
             "via_filter": self.via_filter,
             "via_stash": self.via_stash,
         }
 
         if self.state == "Revoked":
-            log.warning(logdata)
+            log.warning(json.dumps(logdata))
         else:
-            log.info(logdata)
+            log.info(json.dumps(logdata))
 
 
 class CRLiteQuery(object):
@@ -532,7 +537,10 @@ class CRLiteQuery(object):
                 "tbsCertificate"
             ).getComponentByName("serialNumber")
 
-            # Serial numbers are represented big-endian
+            # Serial numbers are represented big-endian.
+            # N.B.: Python strips leading zeroes, so we need to see whether
+            # there might have been one in the serial_bytes. See
+            # CRLiteQueryResult's init method for how that works.
             serial_bytes = int(serial_number).to_bytes(
                 (int(serial_number).bit_length() + 7) // 8,
                 byteorder="big",
