@@ -82,6 +82,11 @@ def main():
         help="Use this specific filter file, ignoring the database",
         type=Path,
     )
+    parser.add_argument(
+        "--check-freshness",
+        help="Set exit code 0 if the database is more than this many hours old",
+        type=int,
+    )
     parser.add_argument(
         "--check-not-revoked",
         help="Set exit code 0 if none of the supplied certificates are revoked",
@@ -108,7 +113,7 @@ def main():
         default=intermediates_collection_prod,
         help="URL to the CRLite records at Remote Settings.",
     )
-    group.add_argument(
+    parser.add_argument(
         "--download-intermediates",
         action="store_true",
         help="Download all intermediate PEM files to the database",
@@ -127,9 +132,9 @@ def main():
     if args.crlite_staging:
         args.crlite_url = crlite_collection_stage
 
-    if args.verbose > 0:
+    if args.verbose > 1:
         logging.basicConfig(level=logging.DEBUG)
-        if args.verbose > 1:
+        if args.verbose > 2:
             from pyasn1 import debug
 
             debug.setLogger(debug.Debug("all"))
@@ -190,6 +195,15 @@ def main():
 
     log.info(f"Status: {intermediates_db}, {crlite_db}")
 
+    if args.check_freshness:
+        freshness_limit = timedelta(hours=args.check_freshness)
+        if crlite_db.age() > freshness_limit:
+            log.error(
+                f"Database age is {crlite_db.age()}, which is larger than {freshness_limit}, "
+                + "aborting!"
+            )
+            sys.exit(1)
+
     query = CRLiteQuery(intermediates_db=intermediates_db, crlite_db=crlite_db)
 
     if not args.files and not args.hosts and not args.hosts_file:
@@ -223,15 +237,15 @@ def main():
             if args.structured:
                 result.log_query_result()
             else:
-                result.print_query_result()
+                result.print_query_result(verbose=args.verbose)
 
             if args.check_not_revoked and result.is_revoked():
                 failures.append(result)
 
     if failures:
-        print(f"{len(failures)} failures logged:")
+        log.error(f"{len(failures)} failures logged:")
         for result in failures:
-            print(result)
+            log.error(result)
         sys.exit(1)
 
 
